;**************************************************************************
;* FileName	: KEYBOARD.INC
;* Date 	: 09.10.2002
;* File version : v1.01
;*
;* Author	: MIKO, Imre
;* Company	:
;* Homepage	: http://majki.fw.hu
;* E-mail	: majki@fw.hu
;*
;*-------------------------------------------------------------------------
;*
;* Description	: Read 4x4 Keyboard matrix
;*		  ==================================
;*		  KEYB_init
;*		  KEYB_get
;*		  KEYB_poll
;*
;*
;*-------------------------------------------------------------------------
;* Needed defines and includes in main-modul:
;* 
;* #define		KEYB_port	PORTB
;*
;*-------------------------------------------------------------------------
;* Other possible defines in main-modul:
;*
;* <none>
;*
;**************************************************************************
;* Revision History:
;*
;* 1.00	  03/09/2000 	Initial Release
;* 1.01   09/10/2002	reformatting source
;*
;**************************************************************************

;***** DEFINES ************************************************************
	ifdef	INC_KEYBOARD
#define		KEYB_bufferfull	KEYB_flags, 0
#define		KEYB_debounce	KEYB_flags, 1
#define		KEYB_waitrel	KEYB_flags, 2
	endif

;***** VARIABLES **********************************************************

	ifdef	INC_KEYBOARD
		cblock
		  KEYB_flags		; flags
		  KEYB_buffer		; keybuffer
		  KEYB_tempbuffer	; temp. keybuffer
		  KEYB_i		; loop counter
		  KEYB_temp		; temp. reg
		endc
	endif


;***** FUNCTIONS **********************************************************
	ifdef	INC_KEYBOARD


;***** KEYB_init **********************************************************
;* Initialize keyboard routines
;* Input : -	        		; 
;* Output: - 				; 
;* Clock : 6  
;**************************************************************************
KEYB_init	bcf	KEYB_bufferfull
		bcf	KEYB_debounce
		bcf	KEYB_waitrel
		clrf	KEYB_buffer
		return


;***** KEYB_get ***********************************************************
;* return with ASCII-code of the last pressed key
;* Input : -             		 
;* Output: w <- ASCII code of key or 0, if key not pressed
;* Clock : 3-18
;**************************************************************************
KEYB_get	btfss	KEYB_bufferfull
		retlw	0

		bcf	KEYB_bufferfull
		bsf	KEYB_waitrel
		
		movf	KEYB_buffer, w
		addlw	LOW KEYB_table		; set CARRY

		movlw	HIGH KEYB_table
		skipnc
		incw
		movwf	PCLATH

		movf	KEYB_buffer, w

		clrf	KEYB_buffer		; KEYB_buffer = -1
		decf	KEYB_buffer, f

		andlw	0Fh
		addwf	PCL,f

KEYB_table	retlw	'0'
		retlw	'1'
		retlw	'2'
		retlw	'3'
		retlw	'4'
		retlw	'5'
		retlw	'6'
		retlw	'7'
		retlw	'8'
		retlw	'9'
		retlw	'A'
		retlw	'B'
		retlw	'C'
		retlw	'D'
		retlw	'E'
		retlw	'F'



;***** KEYB_poll **********************************************************
;* Polling the keyboard (should be call in every 10-50mS)
;* Input : -             		; 
;* Output: - 				; 
;* Clock : 3-124
;**************************************************************************
KEYB_poll	btfsc	KEYB_bufferfull
		return
		
		goto	KEYB_readport

KEYB_p1		btfsc	KEYB_debounce
		goto	KEYB_debo

		btfsc	KEYB_waitrel
		goto	KEYB_waitr

		incf	KEYB_tempbuffer, w
		skipnz
		return	

		movf	KEYB_tempbuffer,w
		movwf	KEYB_buffer
		bsf	KEYB_debounce
		return

		
KEYB_waitr	incf	KEYB_tempbuffer, w	; if (KEYB_tempbuffer==-1) KEYB_debounce=1
		skipnz
		bsf	KEYB_debounce
		return


KEYB_debo	bcf	KEYB_debounce
		movf	KEYB_tempbuffer,w
		subwf	KEYB_buffer,w
		skipz
		return

		btfss	KEYB_waitrel		; if (!KEYB_waitrel) KEYB_bufferfull=1
		bsf	KEYB_bufferfull

		bcf	KEYB_waitrel
		return


;--------------------------------------------------------------------------
; return in KEYB_tempbuffer : 0-15 or -1 
;--------------------------------------------------------------------------
KEYB_readport	bank1
		movlw	11110000b
		movwf	KEYB_port	; keyboard port : IIIIOOOO
		bank0

		movlw	4
		movwf	KEYB_i		
		movlw	00000001b
		movwf	KEYB_temp
		
KEYB_1:			movf	KEYB_temp, w
			movwf	KEYB_port
			nop2
			nop2
			movf	KEYB_port, w
			andlw	11110000b
			jnz	KEYB_getcode

			clrc
			rlf	KEYB_temp, f 

		decfsz	KEYB_i, f
		goto	KEYB_1

		movlw	-1
		movwf	KEYB_tempbuffer

KEYB_endread	bank1
		movlw	11111111b
		movwf	KEYB_port	; keyboard port : IIIIIIII
		bank0

		goto	KEYB_p1

;--------------------------------------------------------------------------
KEYB_getcode	movwf	KEYB_temp

		decf	KEYB_i, w
		andlw	00000011b
		movwf	KEYB_tempbuffer
		clrc	
		rlf	KEYB_tempbuffer, f
		rlf	KEYB_tempbuffer, f

		movlw	-1

KEYB_2:			addlw	1
			setc
			rlf	KEYB_temp, f

		jnc	KEYB_2
				
		andlw	00000011b
		iorwf	KEYB_tempbuffer, f

		goto	KEYB_endread

	endif



;**************************************************************************

